#!/usr/bin/perl -s
#********************************************************************************
#                                                                               *
#                       f2html.pl                                               *
#                                                                               *
#********************************************************************************
# Copyright (C) 2000-2002 by Nikolei Steinhage.   All Rights Reserved.          *
#********************************************************************************
# This program is free software; you can redistribute it and/or                 *
# modify it under the terms of the GNU General Public                           *
# License as published by the Free Software Foundation; either                  *
# version 2 of the License, or (at your option) any later version.              *
#                                                                               *
# This program is distributed in the hope that it will be useful,               *
# but WITHOUT ANY WARRANTY; without even the implied warranty of                *
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
# General Public License for more details.                                      *
#                                                                               *
# You should have received a copy of the GNU Library General Public             *
# License along with this program/script; if not, write to the Free             *
# Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
#********************************************************************************
# Bugs: Doubtless many. Feel free to contact the author, but remember:          *
#       I do not promise anything.                                              *
#       nikolei@linuxsecure.de                                                  *
#********************************************************************************
# Disclaimer: - I do not support Perl, html, or any OS                          *
#             - I do not promise that this program does anything                *
#             - this program may distroy all your data                          *
#             - I cannot promise that this program will not distroy             *
#               all human civilization                                          *
#********************************************************************************
# Das fortwaerende Dasein des Menschengeschlechts ist bloss ein Beweis          *
# der Geilheit desselben.         ATHUR SCHOPENHAUER                            *
# Translation: The continous existence of mankind is only a proof of their      *
# contempt.                                                                     *
#********************************************************************************


use Getopt::Long;

#********************************************************************************
# global variables i need

my @mp3v;                       # directories for depth search
my $htmlfile;                   # html file (at the moment only one)
my $m3ufile;                    # playlistfile
my $dbfile;                     # database file
my @filelist;                   # array to store file and info seperated by the string "@@@"
my $hexfile;                    # file the hextable ist stored in
my %hexhash;                    # hash to store hexreplacements
my $httpaddress;                # address of httpd
my $seperatepath;               # path for seperate html files
my @checkext;                   # string to keep fileext for checks
my $os="unix";                  # default os
my @execs;                      # array for execs
my $depth;                      # depth of dir elements to keep
my %replhash;                   # hash for url replacement
my $replacefile;                # file that contains strings to be replaced at the beginning of
                                # each url; the form is for example:  c:\mp3\A /mp3C and then a new line
my %sum_files=0;                # hash for number of files for every beginning character
my %sum_sizes=0;                # hash for sum of sizes for every character
my $statfile;                   # file for html-statistics
my @sqlinfos;                   # list for sql data
my $sqlfile;                    # file to store sql table in
my %dublicates;                 # to store number of dublicates per file
my %content;                    # content hash for better sorting
my @index;                      # index hash for filenames for better sorting
my @copyfilelist;               # copy of the filelist (used to integrate normal and fast sorting)
my $global_rename_counter=0;


#********************************************************************************
# flags to define for controlling runnig of the program
my $MP3FLAG=0;                  # get mp3info
my $TAGRMFLAG=0;                # remove tags
my $HTTPFLAG=0;                 # correct path and character
my $M3UFLAG=0;                  # write m3u file
my $HTMLFLAG=0;                 # write html file
my $DBFLAG=0;                   # write db file
my $OSFLAG=0;                   # set os, default = unix
my $REMEXTFLAG=0;               # remove file extensions
my $SEPERATEFLAG=0;             # write seperate html file for every character
my $CHECKFLAG=0;                # check for file extension like "mp3"
my $FILESIZEFLAG=0;             # include filesize in html output
my $DEPTHFLAG=0;                # use only last n elements of dir/file
my $REPLACEFLAG=0;              # replace pattern given by replacementfile
my $HEXFLAG=0;                  # replace characters by their hex
my $NOCSFLAG=0;                 # do sorting not case sensitive
my $STATFILEFLAG=0;             # write statistics file
my $SQLFLAG=0;                  # write sql data
my $MKCONFFLAG=0;               # write a config file
my $DUBLICATEFLAG=0;            # check for doblicates of files by their name
my $FASTSORTFLAG=0;             # sort faster (probles when same filename occurs more than once)
my $RENAMEFLAG=0;               # rename mp3 and ogg files using their id3tag
my $RENAMBACKFLAG=0;            # rename files back using the file RENAME.bak
#********************************************************************************
# counter and summaries


#********************************************************************************
# so lets start the program

&Checkmodules;
use MP3::Info;
&Programinfo;
&Mainprog;



#********************************************************************************
# check and load modules needed

sub Checkmodules {

  my $module;  # test perl modules
  
  if(($module=eval{require MP3::Info})==0){
    print STDERR ("\nYou have to install the Perl module 'MP3::Info'.");        
    print STDERR ("\nIt is needed for all mp3 and some ogg vorbis related functions.");        
    print STDERR ("\nSee the 'README' for instructions on how to install it!\n");            
    exit 1;
  }

}




#********************************************************************************
# here is the main prog

sub Mainprog {

  $|=1; # turn off I/O buffering
  

  &Getparameters;

  
  if($RENAMEBACKFLAG==1){&Renamefilesbyorig;}

  if($MKCONFFLAG==1){&Write_Configfile;}

  if($HEXFLAG==1){&Loadhextable($hexfile);}

  if($REPLACEFLAG==1){&Loadreplacementtable($replacefile);}

  &Searchengine;

  &Message("Begin","Sorting filelist");
  &Message("Info","Depending on the number of files sorting may be slow");
  if($FASTSORTFLAG==1){
    &Message("Info","Using fast sorting");
    if($NOCSFLAG==1){@index=sort{ lc($a) cmp lc($b)} @index;}
    else{@index=sort {$a cmp $b} @index;}
  }
  else{
    &Message("Info","Using normal sorting");
    if($NOCSFLAG==1){@filelist=sort{ lc($a) cmp lc($b)}@filelist;}
    else{@filelist=sort(@filelist);}
  }

  if($FASTSORTFLAG==1){
  	@copyfilelist=@index;
  }
  else{
  	@copyfilelist=@filelist;
  }

  if($M3UFLAG==1){&Writem3u($m3ufile);}

  if($HTMLFLAG==1 && $SEPERATEFLAG==0){&Writehtml($htmlfile);}

  if($DBFLAG==1){&Writedb($dbfile);}

  if($SEPERATEFLAG==1 && $HTMLFLAG==0){&Writesephtml($seperatepath);}

  if($STATFILEFLAG==1){&Writestatfile($statfile);}

  if($SQLFLAG==1){&Writesql($sqlfile);}

  if($DUBLICATEFLAG==1){&Printdublicates;}

  if(@execs){
    foreach $execs(@execs){
      &Execsys($execs);
    }
  }


  &Message("Job","done");

}

#********************************************************************************
# Info about the program at program start

sub Programinfo {
    print "\n     f2html.pl Version 0.8 (c)2000-2002 by Nikolei Steinhage\n\n";
}

#********************************************************************************
# just make a output on stdout for sysntax etc.

# already used chars: abcdefghijklmnopqrstxyz (all)

sub Help {
	print <<EOF
f2html.pl (c)2000-2002. Make playlists and html files. Version: 0.8
by Nikolei Steinhage <nikolei\@linuxsecure.de>
Latest version at http://www.linuxsecure.de/index.php?action=55
This program is licensed under Version 2 of the GNU Public License.
See http://www.gnu.org for details.

Usage: perl file2html.pl [-h] [-k] [-n] [[-o (win/unix)] [[-t] <file>] [[-m] <file>]
                         [[-b] <file>] [[-c] <ext> [-c] <ext> ...] [-R] [-B]
                         [[-a] <address>] [-r] [-q] [-i] [-e] [-l] [[-s] <path>]
                         [[-j] <file>] [[-g] <file>]
                         [[-p] <file>] [[-y] <file>] [[-w] <file>]
                         [[-d] <directory1> [-d] <directory2> ...]
			 

	-h, --help                  Show this screen and exit
	-l, --fast                  Use fast sorting (experimental)
	-k, --mkconf                Use an assistant to write a config file
        -o, --os       (win,unix)   Default "unix", otherwise windows
	-n, --dublicates            Check for dublicates of files by their
	                            filename
        -b, --dbfile   <file>       Write database to a file for searching it
        -m, --m3u      <file>       Write a m3u file
        -t, --html     <file>       Write a html file
        -a, --http     <name>       Define the http address for url
        -r, --remove                Remove id3tags
        -i, --mp3info               Use mp3info for html output
        -e, --ext                   Remove file extensions in html output
        -f, --filesize              Use filesize for html output
        -c, --check    <ext>        Select files by their extension(s) (e.g. mp3)
        -z, --skip     <number>     Skip n elements of mount/directories/names
        -p, --conf     <file>       Use a config file
        -w, --utf8     <file>       File with UTF-8 code for replacements in links
        -q, --nocs                  Do sorting not case sensitive
        -j, --statfile <file>       Write statistics to file
        -d, --dir      <directory>  Define the (multiple) directory(ies) the
                                    mp3s are stored in
	-g, --sql      <file>       Filename to store sql table in (only for mp3 and ogg!)
        -y, --replace  <file>       Name of replacement file; in the file use
                                    string_1------>string_2 to transform string_1
                                    into string_2; special characters like a
                                    backslash have to be preceeded by a
                                    backslash "\\"
        -s, --seperate <path>       Write seperate html files for every
                                    character
	-R, --rename                renames mp3 and ogg file using their id3tag
	-B, --renameback	    renames files back using the file RENAME.bak	    
        in the config file it is possible to use the command
        x=, exec=      <param>      Execute system command.
EOF
;

	exit 1;
}

#********************************************************************************
# get the parameters and check them
# here a lot of things have to be done !!!!!!!!!!

sub Getparameters {

  my $error;
  my $conffile;

  $error=&GetOptions(
  "help|h"      => \&Help,
  "mkconf|k"    => \$MKCONFFLAG,
  "os|o:s"      => \$os,
  "dbfile|b:s"  => \$dbfile,
  "m3u|m:s"     => \$m3ufile,
  "html|t:s"    => \$htmlfile,
  "http|a:s"    => \$httpaddress,
  "remove|r"    => \$TAGRMFLAG, # no argument; variable is set to one if true
  "mp3info|i"   => \$MP3FLAG,
  "ext|e"       => \$REMEXTFLAG,
  "seperate|s:s"=> \$seperatepath,
  "dir|d:s"     => \@mp3v,
  "check|c:s"   => \@checkext,
  "filesize|f"  => \$FILESIZEFLAG,
  "conf|p:s"    => \$conffile,
  "skip|z:i"    => \$depth,
  "replace|y:s" => \$replacefile,
  "hexfile|w:s" => \$hexfile,
  "nocs|q"      => \$NOCSFLAG,   # no argument; variale is set to one if true
  "statfile|j:s"=> \$statfile,
  "sql|g:s"     => \$sqlfile,
  "dublicates|n"=> \$DUBLICATEFLAG,
  "fast|l"      => \$FASTSORTFLAG,
  "rename|R"    => \$RENAMEFLAG,
  "renameback|B"=> \$RENAMEBACKFLAG,
  );


  # if a configuration file is specified, then use it!!!
  if($conffile){&Configfile($conffile);}

  # set the flags needed and not set in GetOptions!!!!
  if($os eq "win"){$OSFLAG=1;}
  else{$OSFLAG=0;}                 # default unix else window

  # delete last slash or backslash if existing
  $seperatepath=&Delslash($seperatepath,$OSFLAG);
  foreach $mp3v(@mp3v){$mp3v=&Delslash($mp3v,$OSFLAG);}


  if($httpaddress){$HTTPFLAG=1;}   # set the url to httpaddess for htmlfile, correct slashes for win
  if($hexfile){$HEXFLAG=1;}
  if($dbfile){$DBFLAG=1;}
  if($m3ufile){$M3UFLAG=1;}
  if($htmlfile){$HTMLFLAG=1;}
  if($seperatepath){$SEPERATEFLAG=1;}
  if(@checkext){$CHECKFLAG=1;}
  if($depth){$DEPTHFLAG=1;}
  if($replacefile){$REPLACEFLAG=1;}
#  if($sepdirs){$SEPDIRSFLAG=1;}
  if($statfile){$STATFILEFLAG=1;}
  if($sqlfile){$SQLFLAG=1;}


  if(!@mp3v &&  $MKCONFFLAG==0){
	  warn "\nTry file2html.pl --help |more\n";
	  exit 1;
  }

  if(!$error) {
          &Help;
          exit 1;
  }

}

#********************************************************************************
# search for files through the directories and get the informations needed

sub Searchengine {
  my @statistics;     # to store file statistics
  my $fileinfos;      # to store file infos --> is now a global var!
  my $info;           # to store id3tag info
  my $dircount=0;     # to cout directories
  my $name;           # name of current file
  my $CHECKOK;        # if checkext found then 1
  my @keyofhash;      # keys of hash
  my $sumoffiles;     # number of collected files
  my $temp;           # temporary variable

  if($RENAMEFLAG==0){&Message("Begin","Collecting files");}
  else{&Message("Begin","Collecting files and renaming them using id3 tag");
    if(-e "RENAME.bak"){ 
      &Message("Info","File 'RENAME.bak' already exists ... aborting");
      &Message("Info","Delete or rename 'RENAME.bak' fist,");
      &Message("Info","It's in same folder as 'f2html.pl'.");
      exit 1;
    }
    &Openfile("RENAME.bak",">",RENAME);
  }

  foreach(@mp3v){

    chdir($mp3v[$dircount]);
    opendir(ACDIR,$mp3v[$dircount])||die"cannot open $mp3v[$dircount] !";

INPUT: foreach $name(readdir(ACDIR)){
        $fileinfos="";      # free $fileinfos, because some files might be not mp3

        next INPUT  if $name =~ /^\.|\.\.$/;  # skip upper dirs

        if(-d $name){
          if($OSFLAG == 0){push @mp3v, join '',$mp3v[$dircount] , "/" , $name;}
          else{push @mp3v, join '',$mp3v[$dircount] , "\\" , $name;}
          next INPUT;
        }


        ## a very big/long else environment is following
	## if it is a file and not a dierectory then go on!
        else{

         if($CHECKFLAG==1){
            $CHECKOK=0;
            foreach $checkext(@checkext){
              if($name =~ /\.($checkext)\Z/){
                 $CHECKOK=1;
              }
            }
            next INPUT if($CHECKOK==0);
          }


          if($RENAMEFLAG==1){
	    if($name=~/(.mp3)\Z/i){$name=&Renamefilesbyid3($name,$mp3v[$dircount]);}
  	    if($name=~/(.ogg)\Z/i){$name=&Renamefilesbyid3($name,$mp3v[$dircount]);}
	  }
	  
          @statistics=stat($name);

          if($MP3FLAG==1){
            if($name=~/(.mp3)\Z/i){
              $fileinfos=&Getmp3info($name);  ## -> new: &
    	    }                                 ## added 12.10.2002
	    elsif($name=~/(.ogg)\Z/i){
#	      $fileinfos=&Ogg($name);
	      $fileinfos=&Getogginfo($name,$statistics[7]);
	    }
            else{$fileinfos=" ";}             ## added 12.10.2002
          }

          else{$fileinfos=" ";}


          if($SQLFLAG==1){
            if($name=~/(.mp3)\Z/i || $name=~/(.ogg)\Z/i){
              push @sqlinfos, &Getmp3tag($name,$mp3v[$dircount],$statistics[7],$statistics[9]);
	    }
	  }


          if($TAGRMFLAG==1){
            if($name=~/(.mp3)\Z/i || $name=~/(.ogg)\Z/i){	  
              &Removeid3tag($name);
            }
          }

          if($DUBLICATEFLAG==1){
	     $dublicates{$name}++;
	  }

          if($FASTSORTFLAG==1){
            $temp=$name."".$dircount;
	    push @index, $temp;
            $content{$temp}=join '@@@', $name, $dircount,$statistics[7],$statistics[9],$fileinfos;
	  }
	  else{
            push @filelist , join '@@@', $name,$dircount,$statistics[7],$statistics[9],$fileinfos;
          }


          # new ###############################
          if($NOCSFLAG==1){
             $sum_files{uc(substr($name,0,1))}++;
             $sum_sizes{uc(substr($name,0,1))}+=$statistics[7];
          }
          else{
             $sum_files{substr($name,0,1)}++;
             $sum_sizes{substr($name,0,1)}+=$statistics[7];

          }
#	  print "$name";
        }
    }
    closedir(ACDIR);
    $dircount++;
  }

  @keysofhash = keys %sum_files;
  foreach $keysofhash(@keysofhash){$sumoffiles+=$sum_files{$keysofhash};}
  if($RENAMEFLAG==1){&Message("Info","Renamed $global_rename_counter file(s)");}
  &Message("Info","Collected $sumoffiles file(s)");
  if($RENAMEFLAG==1){&Closefile(RENAME);}
}


#********************************************************************************
# write sql table: &Writesql

sub Writesql {
  my $fname = $_[0];   # name of sql config/table file

  &Openfile($fname,">",SQLFILE);

  &Message("Begin","Writing sql file");

  # the sql table will have the following form:
  #CREATE TABLE mp3table
  #(
  #        name        CHAR(255) NOT NULL,
  #        title       CHAR(30) NULL,
  #        artist      CHAR(30) NULL,
  #        album       CHAR(30) NULL,
  #        year        SMALLINT UNSIGNED NULL,
  #        comment     CHAR(30) NULL,
  #        genre       TINYINT UNSIGNED NULL,
  #        directory   CHAR(255) NOT NULL,
  #        size        INT UNSIGNED NOT NULL,
  #        moditime    INT UNSIGNED NOT NULL,
  #        vbr         CHAR(1) NOT NULL,
  #        bitrate     SMALLINT UNSIGNED NOT NULL,
  #        frequency   TINYINT UNSIGNED NOT NULL,
  #        min         TINYINT UNSIGNED NOT NULL,
  #        sec         TINYINT UNSIGNED NOT NULL
  #);
  #\n";

  foreach $sqlinfos(@sqlinfos){
    print {SQLFILE} "$sqlinfos";
  }

  &Closefile(SQLFILE);
}


#********************************************************************************
# get the mp3 tag:  &Getmp3tag(params)

sub Getmp3tag {
  my $fname = $_[0];  # filename
  my $fdir  = $_[1];  # dirname
  my $fstat7= $_[2];  # statistics[7]
  my $fstat9= $_[3];  # statistics[9]
  my $a;              # temporary variable
  my $b;              # temporary variable
  my $c;              # temporary variable
  my %vorbis_header_info;
  my $time;
  my $min;
  my $sec;


  # hope that your ogg vorbis files also have an mp3tag
  $b=get_mp3tag($fname);

  if($fname=~/(.mp3)\Z/){
     $c=get_mp3info($fname);
  }
  elsif($fname=~/(.ogg)\Z/){
     %vorbis_header_info=&Ogg($fname);
  }



#	%vorbis_header_info=&Ogg($oggfilename);
#	$time=int($oggfilesize/(int($vorbis_header_info{'bitrate_nominal'}/8)));
#	$min=int($time/60);
#	$sec=$time%60;
#	$a=join '',int($vorbis_header_info{'bitrate_nominal'}/1000), " kbits/s, " , $vorbis_header_info{'samplerate'}/1000, " kHz, " ,
#		   $min , "\'\'" . $sec , "\'";



#  $a=join ('\",', "INSERT mp3table (" . $name . $b->{title} . $b->{ARTIST} . b->{ALBUM} .
#                 $b->{YEAR} . $b->{COMMENT} . $b->{GENRE} . $fdir . $fstat7 . $fstat9 .
#		 $c->{VBR} . $c-> {BITRATE} . $c->{FREQUENCY} . $c->{MM} . $c->{SS}. ")\n");

# for windows we need escape character for the backslash
  if($OSFLAG == 1){
    $fdir         =~ s/\\/\\\\/g;
  }

# we have to use escape characters for "'":
# substitute "'" by "\'" for the following variables
  $fname        =~ s/\'/\\\'/g;
  $fdir         =~ s/\'/\\\'/g;
  $b->{title}   =~ s/\'/\\\'/g;
  $b->{ARTIST}  =~ s/\'/\\\'/g;
  $b->{ALBUM}   =~ s/\'/\\\'/g;
  $b->{COMMENT} =~ s/\'/\\\'/g;
  $b->{GENRE}   =~ s/\'/\\\'/g;

  if($fname=~/(.mp3)\Z/){
    $a=("INSERT mp3table VALUES (\'" . $fname . "\',\'" . $b->{title} . "\',\'" . $b->{ARTIST} . "\',\'" .
                   $b->{ALBUM} . "\',\'" . $b->{YEAR} . "\',\'" . $b->{COMMENT} . "\',\'" . $b->{GENRE} .
		   "\',\'" . $fdir . "\',\'" . $fstat7 . "\',\'" . $fstat9 . "\',\'" . $c->{VBR} . "\',\'" .
		   $c-> {BITRATE} . "\',\'" . $c->{FREQUENCY} . "\',\'" . $c->{MM} . "\',\'" . $c->{SS}. "\',\'" . "mpeg 1 layer 3\');\n");
  }
  elsif($fname=~/(.ogg)\Z/){
    $time=int($fstat7/(int($vorbis_header_info{'bitrate_nominal'}/8)));
    $min=int($time/60);
    $sec=$time%60;
    $a=("INSERT mp3table VALUES (\'" . $fname . "\',\'" . $b->{title} . "\',\'" . $b->{ARTIST} . "\',\'" .
                   $b->{ALBUM} . "\',\'" . $b->{YEAR} . "\',\'" . $b->{COMMENT} . "\',\'" . $b->{GENRE} .
		   "\',\'" . $fdir . "\',\'" . $fstat7 . "\',\'" . $fstat9 . "\',\'" . "" . "\',\'" .
		   int($vorbis_header_info{'bitrate_nominal'}/1000) . "\',\'" . $vorbis_header_info{'samplerate'}/1000 . "\',\'" . $min . "\',\'" . $sec. "\',\'" . "ogg vorbis\');\n");
  }


### ---> transform winamp genres
  return $a;

}

#********************************************************************************
# load  unicode table if necessary:  &Loadunicodetable(filename)

sub Loadhextable {
  my $param = $_[0];   # name of hexfile
  my $a;               # temporary variable
  my $b;               # temporary variable
  my $line;            # temporary, line to be read in

  &Openfile($param,"<",HEXFILE);
  &Message("Begin","Loading UTF-8 table");

  while($line=<HEXFILE>){
    chomp($line);
    ($a,$b)=split/ /,$line,2;
    $hexhash{$a}=$b;
  }
  &Closefile(HEXFILE);
}



#********************************************************************************
# correct hex codes in links: &Correcthexlinks(name)

sub Correcthexlinks {
  my $param = $_[0];    # paramter of sub routine
  my $j;                # counter
  my $a="";             # temopary varibale
  my $b="";             # array to return;
  my $lenghofparam=0;   # to mack it faster claculate it once


  $lenghofparam=length($param);
  for($j=0;$j<=$lenghofparam;$j++){
    $a=substr($param,$j,1);
    if($hexhash{$a}){$a=$hexhash{$a};}
#    else{}
    $b=$b.$a;
  }
  return $b;
}

#********************************************************************************
# build replacementhash

sub Loadreplacementtable {
  my $param = $_[0];   # name of file
  my $a;               # temporary variable
  my $b;               # temporary variable
  my $line;            # temporary, line to be read in

  &Openfile($param,"<",REPLFILE);
  &Message("Begin","Loading replacement table");

  while($line=<REPLFILE>){
    chomp($line);
    # a problem might occur if spaces are in the replacementstrings
    ($a,$b)=split/----->/,$line,2;
    $replhash{$a}=$b;
  }

  &Closefile(REPLFILE);

}



#********************************************************************************
# correct links by replacement file: &Replacelink(name)

sub Replacelink {

  my $param = $_[0];   # name of url
  my @y;      # keys of hash
  my $a;

  @y=keys %replhash;


  foreach $y(@y){
    $param =~ s/^$y/$replhash{$y}/ge;
  }

  return $param;

}


#********************************************************************************
# delete file extension:  &Reletefileext(name)

sub Deletefileext {

  my $param = $_[0];   # parameter of sub routine

  $param=~ s/\.*\w*\Z//g;

  return $param;
}


#********************************************************************************
# correct path-names for win if html contains http adress; from / to \
# : &Correctpathname(name)

sub Correctpathnames {
  my $param = $_[0];   # parameter of sub routine

  $param=~ s/\\/\//g;

  return $param;
}

#********************************************************************************
# remove the id3tag: &Removeid3tag(filename)

sub Removeid3tag {
  my $param = $_[0];   # parameter of sub routine

  remove_mp3tag($param);

}


#********************************************************************************
# parse vorbis: &vorbis_parse()
# the original code of vorbis_parse was written by Jens Burkal
# I took it and modified it
sub vorbis_parse{

	my $VORBIS_STREAM_ID = 'vorbis';

	my $vorbisdata = shift;
	my $stream_no = shift;

	my(%vorbis_header_info, $header_type, $stream_id);

	my $offset = 0;

	do {

	($header_type, $stream_id) = unpack("C a6", substr($vorbisdata, $offset, 7));
	$offset += 7;

	# If it is a vorbis-header-package, get basic information
	if (($header_type == 1) and ($stream_id eq $VORBIS_STREAM_ID)) {
		($vorbis_header_info{'version'},
		 $vorbis_header_info{'channels'},
		 $vorbis_header_info{'samplerate'},
		 $vorbis_header_info{'bitrate_upper'},
		 $vorbis_header_info{'bitrate_nominal'},
		 $vorbis_header_info{'bitrate_lower'},
		 $vorbis_header_info{'blocksize'}) = unpack("L C L L L L H2", substr($vorbisdata, $offset, 22));
		$offset += 22;

		# If we don't get the end-of-packet signal (0x01), something must be wrong!
		$offset++;

	}
	# If it is a vorbis-comment-package, get the comments
	elsif (($header_type == 3) and ($stream_id eq $VORBIS_STREAM_ID)) {
		# Get the length of the vendor string
		$vorbis_header_info{'vendor_string_length'} = unpack("L", substr($vorbisdata, $offset, 4));
		$offset += 4;

		# Get the vendor string
		$vorbis_header_info{'vendor_string'} = unpack("a*", substr($vorbisdata, $offset, $vorbis_header_info{'vendor_string_length'}));
		$offset += $vorbis_header_info{'vendor_string_length'};

  		# Get the total number of comments in vorbis-package
		$vorbis_header_info{'no_comments'} = unpack("L", substr($vorbisdata, $offset, 4));
		$offset += 4;

		for (my $i=0; $i < $vorbis_header_info{'no_comments'}; $i++) {
			$vorbis_header_info{'comment_length'}[$i] = unpack("L", substr($vorbisdata, $offset, 4));
			$offset += 4;

			$vorbis_header_info{'comment'}[$i] = unpack("a*", substr($vorbisdata, $offset, $vorbis_header_info{'comment_length'}[$i]));
			$offset += $vorbis_header_info{'comment_length'}[$i];

		}
		$offset++;

	} else {
		return 0;
	}

	} until ($offset == length($vorbisdata));

	return %vorbis_header_info;
}



#********************************************************************************
# Ogg: &Ogg(filename)
# the original code of vorbis_parse was written by Jens Burkal
# I took it and modified it
sub Ogg{

	my $oggfilename = $_[0];
	my $self = {};
	my %vorbis_header_info;

	&Openfile($oggfilename,"<","OGGFILE");

	my (%headerinfo, $oggheader, $count, %ogginfo, $seg_table, $total_package_length, $ogg_payload, @package_lengths, %page_id);

	$self->{'no_log_bitstreams'} = 0;

	my ($CAPTURE_PATTERN) = chr(0x4f). chr(0x67). chr(0x67). chr(0x53);	# Capture pattern we want to match: OggS

	$count = read(OGGFILE, $oggheader, 27);
	($headerinfo{'capture_pattern'},
	 $headerinfo{'stream_version'},
	 $headerinfo{'header_type'},
	 $headerinfo{'PCM_abs_pos'},
	 $headerinfo{'stream_serial'},
	 $headerinfo{'page_seq'},
	 $headerinfo{'page_crc'},
	 $headerinfo{'page_segments'}) = unpack("a4 c c H16 i i H8 C", $oggheader);


	# If the page is the start of a logical bitstream, let's save the serial-number and other usefull info.
	if ($headerinfo{'header_type'} == 2) {
		$self->{'ogg_serial'}[$self->{'no_log_bitstreams'}] = $headerinfo{'stream_serial'};
		$self->{'ogg_version'}[$self->{'no_log_bitstreams'}] = $headerinfo{'stream_version'};

	# To keep track of which bitstream (based on serial) is which, save it's number in an array.
		$page_id{$headerinfo{'stream_serial'}} = $self->{'no_log_bitstreams'};
		$self->{'no_log_bitstreams'}++;
	}

	# Read the segmentation table for the page.
	$count = read(OGGFILE, $seg_table, $headerinfo{'page_segments'});

	# Unpack and calculate the total length of the data in the packages.
	@package_lengths = unpack("C*",$seg_table);

	for (my $i = 0, $total_package_length = 0; $i < @package_lengths; $i++) {
		$total_package_length += $package_lengths[$i];
	}

	# Read the multimedia-data
	$count = read(OGGFILE, $ogg_payload, $total_package_length);

	%vorbis_header_info=vorbis_parse($ogg_payload, $page_id{$headerinfo{'stream_serial'}});

	return %vorbis_header_info;
}

#********************************************************************************
# get ogginfo: &Getogginfo(filename, filesize)

sub Getogginfo {
	my $oggfilename = $_[0];
	my $oggfilesize = $_[1];
	my %vorbis_header_info;
	my $a;                        # temporary variable to store fileinfo
	my $min;                      # minutes tracklength
	my $sec;                      # seconds tracklength
	my $time;                     # tmporary time variable

	%vorbis_header_info=&Ogg($oggfilename);
	$time=int($oggfilesize/(int($vorbis_header_info{'bitrate_nominal'}/8)));
	$min=int($time/60);
	$sec=$time%60;
	$a=join '',int($vorbis_header_info{'bitrate_nominal'}/1000), " kbits/s, " , $vorbis_header_info{'samplerate'}/1000, " kHz, " ,
		   $min , "\'\'" . $sec , "\'";
	return $a;

}

#********************************************************************************
# get mp3info: &Getmp3info(filename)

sub Getmp3info {
  my $param = $_[0];   # filename
  my $a;               # temporary variable to store fileinfos
  my $info;            # get mp3info

  if($info=get_mp3info($param)||warn"\nfile $param not mp3 file or not ok !!!"){
    if ($info->{VBR} == 1){
      # here you can modify the string to adjust the output for the html file
      $a=join '',$info->{BITRATE} , " kbits/s VBR, " , $info->{FREQUENCY} , " kHz, " ,
          $info->{MM} , "\'\'" . $info->{SS} , "\'";
    }
    else{
      # here you can modify the string to adjust the output for the html file
      $a=join '',$info->{BITRATE} , " kbits/s, " , $info->{FREQUENCY} , " kHz, " ,
          $info->{MM} , "\'\'" . $info->{SS} , "\'";
    }
  }

  return $a;
}


#********************************************************************************
# open filehandle: &Openfile(filename,mode,filehandle)

sub Openfile {
   my $filename = $_[0];       # name of file to open
   my $filemode = $_[1];       # mode: read, write, append etc.
   my $filehandle = $_[2];     # name of filehandle

   open($filehandle, "$filemode $filename")||die"cannot open $filename!";

}



#********************************************************************************
# close filehandle: &Closefile(handle);

sub Closefile {
   my $filehandle = $_[0];     # name of filehandle

   close($filehandle)||die"cannot close $filehandle!";
}

#********************************************************************************
# write m3u file: &Writem3u(filename)

sub Writem3u {

  my $param = $_[0];               # name of file to write
  my $datastring="";               # string to store the m3u records
  my $sortfilename;
  my $position;
  my $filesize;
  my $modificationtime;
  my $fileinfos;


  &Openfile($param,">",M3UFILE);
  &Message("Begin","Preparing m3u file");

  if($OSFLAG==0){
    foreach $copyfilelist(@copyfilelist){
      if($FASTSORTFLAG==1){
         ($sortfilename, $position, $filesize, $modificationtime, $fileinfos) = split/@@@/,$content{$copyfilelist},5;
      }
      else{
         ($sortfilename, $position, $filesize, $modificationtime, $fileinfos) = split/@@@/,$copyfilelist,5;
      }
      $datastring=join '',$datastring,"$mp3v[$position]/$sortfilename\n";
    }
  }
  else{
    foreach $copyfilelist(@copyfilelist){
      if($FASTSORTFLAG==1){
         ($sortfilename, $position, $filesize, $modificationtime, $fileinfos) = split/@@@/,$content{$copyfilelist},5;
      }
      else{
         ($sortfilename, $position, $filesize, $modificationtime, $fileinfos) = split/@@@/,$copyfilelist,5;
      }
      $datastring=join '',$datastring,"$mp3v[$position]\\$sortfilename\n";
    }
  }
  print {M3UFILE} "$datastring";
  &Message("Begin","Writing m3u file");
  &Closefile(M3UFILE);
}


#********************************************************************************
# write db file: &Writedb(filename)

sub Writedb {
  my $param = $_[0];               # name of file to write
  my $datastring="";               # string to store the m3u records
  my $sortfilename;
  my $position;
  my $filesize;
  my $modificationtime;
  my $fileinfos;

  &Openfile($param,">",DBFILE);
  &Message("Begin","Preparing db file");


  foreach $filelist(@filelist){
    if($FASTSORTFLAG==1){
      ($sortfilename, $position, $filesize, $modificationtime, $fileinfos) = split/@@@/,$content{$copyfilelist},5;
    }
    else{
      ($sortfilename, $position, $filesize, $modificationtime, $fileinfos) = split/@@@/,$filelist,5;
    }
    $datastring=$datastring . join '@@@',$mp3v[$position],$sortfilename,"$modificationtime\n";
  }

  &Message("Begin","Writing db file");
  print {DBFILE} "$datastring";
  &Closefile(DBFILE);
}

#********************************************************************************
# write one html file or

sub Writehtml {

  my $param = $_[0];            # name of file to write
  my $urlname;                  # url name that can be corrected
  my $datastring="";            # string to store the m3u records
  my $shownname;                # name that is visible
  my $firstchar=" ";            # need a character to compare the first char of exery file with
  my $FTFLAG=0;                 # flag
  my $sunoffiles=0;             # sum of files
  my $sumofmegs=0;              # sum of filesizes
  my $keyofhash=0;              # keys for the hashes sum_file and sum_sizes
  my $last=0;                   # counter
  my $sortfilename;
  my $position;
  my $filesize;
  my $modificationtime;
  my $fileinfos;

  &Message("Begin","Preparing html file");

  &Openfile($param,">",HTMLFILE);


  @keysofhash = keys %sum_files;
  foreach $keysofhash(@keysofhash){$sumoffiles+=$sum_files{$keysofhash};}
  &Message("Info","Progress");
  print "            0------------------50----------------100%\n";
  print "            ";

  $datastring=join '',$datastring,&Htmlheader;


  foreach $copyfilelist(@copyfilelist){

    if($FASTSORTFLAG==1){
      ($sortfilename,$position, $filesize, $modificationtime, $fileinfos) = split/@@@/,$content{$copyfilelist},5;
    }
    else{
      ($sortfilename, $position, $filesize, $modificationtime, $fileinfos) = split/@@@/,$copyfilelist,5;
    }

    $urlname=$mp3v[$position];

    if($REPLACEFLAG==1){$urlname=&Replacelink($urlname)};

    if($OSFLAG==1){$urlname=&Correctpathnames(join '',$urlname,"\\",$sortfilename);}
    else{$urlname=join '',$urlname,"/",$sortfilename;}

    if($DEPTHFLAG==1){$urlname=&Keeppath($urlname,$depth);}

    if($HEXFLAG==1){$urlname=&Correcthexlinks($urlname);}

    if($HTTPFLAG==1){$urlname=join '',$httpaddress,"/",$urlname;}

    if($REMEXTFLAG==1){$shownname=&Deletefileext($sortfilename);}
    else{$shownname=$sortfilename;}

    if($FILESIZEFLAG==1){$filesize=join '',int($filesize/1000),"kb";}
    else{$filesize=" ";}

    if($NOCSFLAG==1 & lc(substr($sortfilename,0,1)) ne lc($firstchar)){
      $firstchar=substr($sortfilename,0,1);
      $datastring=join '',$datastring,&Htmlchange(uc($firstchar));
    }
    elsif($NOCSFLAG==0 & substr($sortfilename,0,1) ne $firstchar){
     $firstchar=substr($sortfilename,0,1);
     $datastring=join '',$datastring,&Htmlchange($firstchar);
    }
   $datastring= join '',$datastring,&Htmlbody($urlname,$shownname,$filesize,$fileinfos);

   $last++;
   if($last>=$sumoffiles/40){
     print "#";
     $last=0;
   }
  }

  print "#\n";

  foreach $keysofhash(@keysofhash){$sumofmegs+=$sum_sizes{$keysofhash};}

  $datastring= join '',$datastring,&Htmlfooter($sumoffiles,int($sumofmegs/1000000),&Date);

  &Message("Begin","Writing html file");
  print {HTMLFILE} "$datastring";
  &Closefile(HTMLFILE);
}

#********************************************************************************
# here is the definition of the html header

sub Htmlheader {

   my @para = $_[0];          # firstchar not needed here
   my $datastring="";         # this variable will contain the html code for the header
   my @y;                     # values of hash

   @y = keys %sum_files;

   $datastring = ("<html><body><a name=top>");

   foreach $y(@y){
      $datastring = join '', $datastring, "<a href=\"\#".$y."\">".$y."</a>";
   }

   $datastring = join '', $datastring, "<br>";

   return $datastring;
}


#********************************************************************************
# here is the definition of the html footer

sub Htmlfooter {

   my @params = @_;           # parameters given to this function like sum of files, date etc.
   my $datastring="";         # this variable will contain the html code for the header

   $datastring = ("
      <BR><P><a href=\"\#top\">back to top</a>
      </P>Sum of files is $params[0].
      <br>Sum of megabytes is $params[1].
      <br>Generated by f2html.pl on $params[2],
      (c)2000-2002 Nikolei Steinhage </body></html>
    ");

   return $datastring;

}

#********************************************************************************
# here is the definition of the html char change: &Htmlchange(char)

sub Htmlchange {

   my $param = $_[0];         # charcater
   my $datastring="";         # this variable will contain the html code for the header

   $datastring=("<br><P><a href=\"\#top\">back to top</a></P><br><P><a name=$param>$param</a></P>\n");

   return $datastring;

}

#********************************************************************************
# here is the definition of the html body: &Htmlbody(vars)

sub Htmlbody {

   my @params = @_;           # paramteters like filename, http etc.
   my $datastring="";         # this variable will contain the html code for the header

   $datastring=("<br><a href=\"$params[0]\">$params[1]</a> $params[2] $params[3]\n");

  return $datastring;

}

#********************************************************************************
# time: &Date

sub Date {

  my @months = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
  my ($sec,$min,$hour,$mday,$mon,$yearcomp) = localtime(time);
  my $time = sprintf("%02d/%s/%d, %02d:%02d:%02d",
                 $mday, $months[$mon], $yearcomp + 1900,
                 $hour, $min, $sec);

  return $time;

}

#********************************************************************************
# write seperate html file for every chart: &Writesephtml(path)
# at the moment this sub is a little bit buggy!!! the misc category dos not
# work very well. so you have to wait ;-) stay tune.

sub Writesephtml {

  my $htmlpath=$_[0];           # path to write html files: a.html, b.html .....
  my $urlname;                  # url name that can be corrected
  my $datastring="";            # string to store the m3u records
  my $shownname;                # name that is visible
  my $firstchar=" ";            # need a character to compare the first char of exery file with
  my $FTFLAG=0;                 # flag
  my $keyofhash=0;              # keys for the hashes sum_file and sum_sizes
  my $last=0;                   # counter
  my $sortfilename;
  my $position;
  my $filesize;
  my $modificationtime;
  my $fileinfos;


  &Message("Begin","Preparing and writing seperate html files");

  @keysofhash = keys %sum_files;
  foreach $keysofhash(@keysofhash){$sumoffiles+=$sum_files{$keysofhash};}
  &Message("Info","Progress");
  print "            0------------------50----------------100%\n";
  print "            ";


  $datastring=$datastring.&Htmlheader;

  foreach $copyfilelist(@copyfilelist){

    if($FASTSORTFLAG==1){
      ($sortfilename, $position, $filesize, $modificationtime, $fileinfos) = split/@@@/,$content{$copyfilelist},4;
    }
    else{
      ($sortfilename, $position, $filesize, $modificationtime, $fileinfos) = split/@@@/,$copyfilelist,5;    
    }

    $urlname=$mp3v[$position];

    if($REPLACEFLAG==1){$urlname=&Replacelink($urlname)};

    if($OSFLAG==1){$urlname=&Correctpathnames(join '',$urlname,"\\",$sortfilename);}
    else{$urlname=join '',$urlname,"/",$sortfilename;}

    if($DEPTHFLAG==1){$urlname=&Keeppath($urlname,$depth);}

    if($HEXFLAG==1){$urlname=&Correcthexlinks($urlname);}

    if($HTTPFLAG==1){$urlname=$httpaddress."/".$urlname;}

    if($REMEXTFLAG==1){$shownname=&Deletefileext($sortfilename);}
    else{$shownname=$sortfilename;}

    if($FILESIZEFLAG==1){$filesize=join '',int($filesize/1000),"kb";}
    else{$filesize=" ";}

    if(($NOCSFLAG==1 & uc(substr($sortfilename,0,1)) ne uc($firstchar))| 
       ($NOCSFLAG==0 & substr($sortfilename,0,1) ne $firstchar)){


     if($firstchar ne " "){
       if($NOCSFLAG==1){$datastring=join '', $datastring,&Htmlfooter($sum_files{uc($firstchar)},int($sum_sizes{uc($firstchar)}/1000000),&Date);}
       else{$datastring=join '', $datastring,&Htmlfooter($sum_files{$firstchar},int($sum_sizes{$firstchar}/1000000),&Date);}
    
#        $datastring=join '', $datastring,&Htmlfooter($sumoffiles,int($sumofmegs/1000000),&Date);       
        print {HTMLFILE} $datastring;
        close(HTMLFILE);
        $datastring="";
#        $sumoffiles=0;
#        $sumofmegs=0;
      }

     if($NOCSFLAG==1){$firstchar=uc(substr($sortfilename,0,1));}
     else{$firstchar=substr($sortfilename,0,1);};
        
     if($NOCSFLAG==1){
        if($OSFLAG==1){&Openfile($htmlpath."\\".uc($firstchar).".html",">",HTMLFILE);}
        else{&Openfile($htmlpath."/".uc($firstchar).".html",">",HTMLFILE);}
        $datastring=&Sephtmlheader(uc($firstchar));
     }
     else{
        if($OSFLAG==1){&Openfile($htmlpath."\\".$firstchar.".html",">",HTMLFILE);}
        else{&Openfile($htmlpath."/".$firstchar.".html",">",HTMLFILE);}
        $datastring=&Sephtmlheader($firstchar);
     }


   }
    $datastring=join '', $datastring,&Htmlbody($urlname,$shownname,$filesize,$fileinfos);
    
   $last++;
   if($last>=$sumoffiles/40){
     print "#";
     $last=0;
   }


  }

  if($NOCSFLAG==1){$datastring=join '', $datastring,&Htmlfooter($sum_files{uc($firstchar)},int($sum_sizes{uc($firstchar)}/1000000),&Date);}
  else{$datastring=join '', $datastring,&Htmlfooter($sum_files{$firstchar},int($sum_sizes{$firstchar}/1000000),&Date);}

  print "#\n";

#  $datastring=join '', $datastring,&Htmlfooter($sumoffiles,int($sumofmegs/1000000),&Date);
#  $datastring=join '', $datastring,&Htmlfooter($sum_files{$firstchar},int($sum_sizes{$firstchar}/1000000),&Date);

#  &Message("Begin","Writing html files");
  print {HTMLFILE} "$datastring";
  &Closefile(HTMLFILE);

}


#********************************************************************************
# write html header for seperate html files: &Sephtmlheader(char)
# $datastring=&Sephtmlheader($firstchar);

sub Sephtmlheader {

  my $param=$_[0];      # parameter: should be $firstchar
  my $datastring="";    # datastring to store html code in

  $datastring=("
    <html>
    <body>
    <p>$param</P>
  ");

  return $datastring;

}


#********************************************************************************
# rename files by original name

sub Renamefilesbyorig {
  my $line;
  my @temp;
  my $b=0;

  &Message("Begin","Renaming files by their original name using backup in RENAME.bak");
  &Openfile("RENAME.bak","<",RENAME);
  # when writing to RENAME.bak it has to be checkt, whether it already exists!!!!! 

  while($line=<RENAME>){
    chomp($line);
    @temp=split/----->/,$line,2;
    (rename $temp[1], $temp[0])||&Message("Info","File $temp[1] does not exist");
    $b++
  }    
  &Message("Info","$b file(s) renamed");  
  &Closefile(RENAME);

}


#********************************************************************************
# rename files by id3tag

sub Renamefilesbyid3 {
    my $temp=$_[0]; # name of file
    my $tempdir=$_ [1]; # name of dir 
    my $b;
    my @title;
    my $newfilename; # to return the old name if no renaming took place
    my $c=2; 
    
    $b=get_mp3tag($temp);
#    if($b->{ARTIST}!='' && $b->{TITLE}!=''){    
    if($b->{ARTIST} && $b->{TITLE}){    


	$title[0]=$b->{ARTIST};
	if($b->{ALBUM}){
	  $title[1]=" - ".$b->{ALBUM};
	  $c++;
	}
	if($b->{TRACKNUM}){
	    if(($b->{TRACKNUM})<10){$b->{TRACKNUM}="0".int($b->{TRACKNUM});}	
	    $title[2]=" - ".$b->{TRACKNUM};
	    $c++;
	}

	$title[3]=" - ".$b->{TITLE};
        $newfilename=join '',@title,substr($temp,length($file)-4,4);	

        if($temp=~tr/-//>=$c){
	  &Message("Info","File '$temp' seems to contain as much info as id3 tag ... skip it");  			
	}
	elsif($temp=~$newfilename){
	  &Message("Info","File '$temp' has valid name ... skip it");  		
	}
        elsif(-e $newfilename){
	  &Message("Info","File '$newfilename' already exists ... skip it");  	
        }
        else{
	  if($OSFLAG==1){
	    &Storefilenames($tempdir."\\".$temp,$tempdir."\\".$newfilename);
	  }
	  else{
	    &Storefilenames($tempdir."/".$temp,$tempdir."/".$newfilename);	  
	  }

          (rename $temp, $newfilename)||&Message("Info","Cannot rename $temp in $tempdir"); 
	  $global_rename_counter++;
        }  
   }
   else{
     $newfilename=$temp; # to return at least the oldd name of the file
   }
 
  return $newfilename;
   
}


#********************************************************************************
# &Storefilenames($old,$new)

sub Storefilenames {
    
    my @temp = @_;
        # do not buffer these thing! here slower means safer!    
    print {RENAME} "$temp[0]----->$temp[1]\n";

}


#********************************************************************************
# check for dublicates of filenames and print them out

sub Printdublicates {
  my @temp;
  my @z;

  &Message("Begin","Counting dublicates");

  @z=keys %dublicates;
  foreach $z(@z){
    if($dublicates{$z}>1){
      push @temp, join'',$z." ".$dublicates{$z}." times";
    }
  }

  &Message("Info","Dublicates found:");
  if(scalar(@temp)==0){
  print "            none\n";
  }
  else{
    foreach $temp(@temp){
        print "            $temp\n";
    }
  }
  &Message("Info","Look into your html output for the exact location of these files");

}

#********************************************************************************
# read parameters from config file: &Configfile(filename)

sub Configfile {

  my $param = $_[0];     # name of config file
  my $line;              # one line of file

  &Openfile($param,"<",CONFFILE);
  while($line=<CONFFILE>){
    chomp($line);
    my(@value)=split/=/,$line;

    SWITCH:{

        if($value[0] =~ /^(\#)/){    # skip any comment line
          last SWITCH;
        }

        elsif($value[0] =~ /^(o)$/ or $value[0] =~ /^(os)/){
          $os=$value[1];
          last SWITCH;
        }

        elsif($value[0] =~ /^(b)$/ or $value[0] =~ /^(dbfile)/){
          $dbfile=$value[1];
          last SWITCH;
        }

        elsif($value[0] =~ /^(m)$/ or $value[0] =~ /^(m3u)/){
          $m3ufile=$value[1];
          last SWITCH;
        }

        elsif($value[0] =~ /^(t)$/ or $value[0] =~ /^(html)/){
          $htmlfile=$value[1];
          last SWITCH;
        }

        elsif($value[0] =~ /^(a)$/ or $value[0] =~ /^(http)/){
          $httpaddress=$value[1]; 
          last SWITCH;
        }

        elsif($value[0] =~ /^(r)$/ or $value[0] =~ /^(remove)/){
          $TAGRMFLAG=1; 
          last SWITCH;
        }

        elsif($value[0] =~ /^(i)$/ or $value[0] =~ /^(mp3info)/){
          $MP3FLAG=1; 
          last SWITCH;
        }

        elsif($value[0] =~ /^(s)$/ or $value[0] =~ /^(seperate)/){
          $seperatepath=$value[1]; 
          last SWITCH;
        }

        elsif($value[0] =~ /^(c)$/ or $value[0] =~ /^(check)/){
#C          $checkext=$value[1]; 
          push @checkext, $value[1];
          last SWITCH;
        }

        elsif($value[0] =~ /^(f)$/ or $value[0] =~ /^(filesize)/){
          $FILESIZEFLAG=1; 
          last SWITCH;
        }

        elsif($value[0] =~ /^(d)$/ or $value[0] =~ /^(dir)/){
          push @mp3v, $value[1]; 
          last SWITCH;
        }

        elsif($value[0] =~ /^(e)$/ or $value[0] =~ /^(ext)/){
          $REMEXTFLAG=1; 
          last SWITCH;
        }

        elsif($value[0] =~ /^(x)$/ or $value[0] =~ /^(exec)/){
          push @execs, $value[1]; 
          last SWITCH;
        }

        elsif($value[0] =~ /^(z)$/ or $value[0] =~ /^(skip)/){
          $depth=$value[1]; 
          last SWITCH;
        }

        elsif($value[0] =~ /^(y)$/ or $value[0] =~ /^(replace)/){
          $replacefile=$value[1]; 
          last SWITCH;
        }

        elsif($value[0] =~ /^(w)$/ or $value[0] =~ /^(utf8)/){
          $hexfile=$value[1]; 
          last SWITCH;
        }

        elsif($value[0] =~ /^(q)$/ or $value[0] =~ /^(nocs)/){
          $NOCSFLAG=1; 
          last SWITCH;
        }

        elsif($value[0] =~ /^(j)$/ or $value[0] =~ /^(statfile)/){
          $STATFILEFLAG=1; 
          $statfile=$value[1];
          last SWITCH;
        }

	elsif($value[0] =~ /^(g)$/ or $value[0] =~ /^(sql)/){
	  $SQLFLAG=1;
	  $sqlfile=$value[1];
	  last SWITCH;
	}

	elsif($value[0] =~ /^(n)$/ or $value[0] =~ /^(dublicates)/){
	  $DUBLICATEFLAG=1;
	  last SWITCH;
	}

	elsif($value[0] =~ /^(l)$/ or $value[0] =~ /^(fast)/){
          $FASTSORTFLAG=1;
	  last SWITCH;
	}

	elsif($value[0] =~ /^(R)$/ or $value[0] =~ /^(rename)/){
          $RENAMEFLAG=1;
	  last SWITCH;
	}

	elsif($value[0] =~ /^(B)$/ or $value[0] =~ /^(renameback)/){
          $RENAMEBACKFLAG=1;
	  last SWITCH;
	}


	else{ 
	  &Message("Info","Do not know command \"$value[0]\"");
	  last SWITCH;
	}


    } # end of SWITCH 

  }

  &Closefile(CONFFILE);

}

#********************************************************************************
# execute system commmands: &Execsys(parameter)

sub Execsys {

  my  $param  = $_[0];    # system command

  &Message("Executing",$execs);
  system($param);
}

#********************************************************************************
# only keep n paths: &Keeppath(path,depth)

sub Keeppath {

  my $pathname  = $_[0];    # pathname
  my $depth     = $_[1];    # depth
  my $tempname="";          # temorary variable
  my @elements;             # elements of tempname
  my $elementcount;         # count elemets
  my $i;                    # simple counter

  @elements=split /\//g,$pathname;
  $elementcount=@elements;
  for($i=$depth+1;$i<$elementcount;$i++){
    $tempname=join '/',$tempname,$elements[$i];
  }
  
  return $tempname;
}

#********************************************************************************
# give out a message on stderr: &Message(1st,2nd)

sub Message {
   my $category = $_[0];  # category
   my $message  = $_[1];  # messageinfo

   printf {STDERR} "%10s: %s\n",$category,$message;  

}

#********************************************************************************
# delete / or \ in patname for configs: &Delslash(path,osflag)

sub Delslash {
   my $pathname=$_[0];
   my $os      =$_[1];

   if($os==0){$pathname =~ s/\/\Z//;}
   else{$pathname =~ s/\\\Z//;}

   return $pathname;
}


#********************************************************************************
# write a statistics file: &Writestatfile(filename);

sub Writestatfile {

    my $file=$_[0];   # filename
    my $datastring="";
    my @y;
    my $sum=0;    
    my $temp;

    @y = keys %sum_files;

    &Message("Begin","Writing statistics file");
    &Openfile($file,">",STATOUT);
    $datastring=join '',$datastring, "<html><body><center><b><u>SOME STATISTICS</u></b><br>&nbsp;<br>&nbsp;<br>";


    ############## number of files per character
    $datastring=join '',$datastring, "<b>NUMBER OF FILES PER CHARACTER</b><br><table border=1><tr>";
    foreach $y(@y){
      $datastring=join '',$datastring, "<td>".$y."</td>";
    }
    $datastring=join '',$datastring, "</tr><tr>";
    foreach $y(@y){
      $datastring=join '',$datastring, "<td>".$sum_files{$y}."</td>";
    }
    $datastring=join '',$datastring, "</tr></table><br>";
    foreach $y(@y){
      $sum+=$sum_files{$y};
    }
    $datastring=join '',$datastring, "<br>&nbsp;<br>";
    ###############


    ############## percentage of files per character
    $datastring=join '',$datastring, "<b>PERCENTAGE OF FILES PER CHARACTER</b><br><table border=1><tr>";
    foreach $y(@y){
      $datastring=join '',$datastring, "<td>".$y."</td>";
    }
    $datastring=join '',$datastring, "</tr><tr>";
    foreach $y(@y){
      $temp=sprintf "%5.2f",(($sum_files{$y}/$sum)*100);
#      $datastring=join '',$datastring, "<td>". (($sum_files{$y}/$sum)*100) ."</td>"; # $sum has bee built before
      $datastring=join '',$datastring, "<td>$temp</td>"; # $sum has bee built before
    }
    $datastring=join '',$datastring, "</tr></table><br>";
    $datastring=join '',$datastring, "Overall there are ".$sum." files.<br>&nbsp;<br>";
    ###############


    ############## sizes of files per character
    $datastring=join '',$datastring, "<b>SUM OF SIZES PER CHARACTER IN MEGA BYTES</b><br><table border=1><tr>";
    foreach $y(@y){
      $datastring=join '',$datastring, "<td>".$y."</td>";
    }
    $datastring=join '',$datastring, "</tr><tr>";
    foreach $y(@y){
      $datastring=join '',$datastring, "<td>".int($sum_sizes{$y}/1000000)."</td>";
    }
    $datastring=join '',$datastring, "</tr></table><br>";
    foreach $y(@y){
      $sum+=$sum_sizes{$y};
    }
    $datastring=join '',$datastring, "<br>&nbsp;<br>";
    ###############


    ############## percentage of sizes of files per character
    $datastring=join '',$datastring, "<b>PERCENTAGE OF SIZES PER CHARACTER</b><br><table border=1><tr>";
    foreach $y(@y){
      $datastring=join '',$datastring, "<td>".$y."</td>";
    }
    $datastring=join '',$datastring, "</tr><tr>";
    foreach $y(@y){
       $temp=sprintf "%5.2f",(($sum_sizes{$y}/$sum)*100);
#      $datastring=join '',$datastring, "<td>". int(($sum_sizes{$y}/$sum)*100)."</td>"; # $sum has bee built before
      $datastring=join '',$datastring, "<td>$temp</td>"; # $sum has bee built before
    }
    $datastring=join '',$datastring, "</tr></table><br>";
    $datastring=join '',$datastring, "Overall there are ".int($sum/1000000)." mb of files.<br>&nbsp;<br>";
    ###############



    $datastring=join '',$datastring, "<br>Generated by f2html.pl on ".&Date.", (c)2000-2002 Nikolei Steinhage </center></body></html>"
;


    print {STATOUT} "$datastring";


    &Closefile(STATOUT);

}

#********************************************************************************
# print a line: &Line();
sub Line {
  print "\n----------------------------------------------------------------------\n";
  print "----------------------------------------------------------------------\n";
}


#********************************************************************************
# write a configfile: Write_Configfile();
sub Write_Configfile {

  my $x;       # temporary variable
  my @configs; # array for lines of configfile!
  my $param;   # filename

  print "\nThis is a tool that will help you to write a custom";
  print "\nconfiguration file for f2html.pl.";
  print "\nIf you want to skip an option, just press RETURN";
  print "\nTo abort, press CTRL-c";
  print "\n\n\nPress Return to continue ....";
  $x=<STDIN>;

  &Line();

  print "\nGive the name (and the absolute path if necessary)";
  print "\nfor/of the config file:";
  chomp($param=<STDIN>);
  if(!$param){
    print "\nNo filename was given!";
    exit 1;
  }

  push @configs, "# This is a config file for f2html generated on ".&Date;
  push @configs, "# I hope that this little proggy is useful for you.";

  &Line();

  # --os, specify the os
  print "\nYou can specify the operating system perl is running on.";
  print "\nFor example:";
  print "\nIf you want to use Windows (whatever version): win";
  print "\nOtherwise just press RETURN.";
  print "\nDefine your operating system or use se default (unix):";
  chomp($x=<STDIN>);
  if($x=~/^(win)/){
    push @configs,"# operating system to be used";
    push @configs,"os=win";
    print "\n------> You are using Windows.\n";
  }
  else{print "\n-----> You are using Linux or so.\n";}

  &Line();

  # --dir, specify directory
  $x="yes";
  while($x=~ /^(y)|(yes)$/){
    print "\nYou can specify a directory to search recursively for files.";
    print "\nFor example:";
    print "\n    For Windows: c:\\music";
    print "\n    For Linux: /home/nikolei/music";
    print "\nGive the absolute path of the directory:";
    chomp($x=<STDIN>);
    if($x){
      push @configs,"# a directory to search in recursively for files";
      push @configs, join '',"dir=".$x;
    }
    print "\nDo you want to specify an other directory? (yes,no):";
    chomp($x=<STDIN>);
  }
  
  &Line();
  
  # --html, specify html file
  print "\nYou can specify a html file to store your file information in.";
  print "\nIf you want to write a seperate html file for every first";
  print "\ncharacter of your file names, then just press RETURN";
  print "\nFor example:";
  print "\n     For Windows: c:\\windows\\Desktop\\myfiles.html";  
  print "\n     For Linux: /home/nikolei/myfiles.html";
  print "\nGive the name of the html file and it's absolute path:";
  chomp($x=<STDIN>);
  if($x){
    push @configs,"# a html file into which the file information is written";
    push @configs, join'',"html=".$x;
  }
  else{print "\n------> You have not specified any html file.\n";}

  &Line();

  # --seperate
  print "\nYou can specify a path to store your file information in.";
  print "\nThe program will write a seperate html file for every first";
  print "\ncharacter of your file names.";
  print "\nFor example:";
  print "\n     For Windows: c:\\windows\\Desktop\\myfiles.html";  
  print "\n     For Linux: /home/nikolei/myfiles.html";
  print "\nIf you have already defined a html file (option before!)";
  print "\nto store all file informations in, the just say no here";
  print "\nor press RETURN.";
  print "\nGive the name of the directory and it's absolute path:";
  chomp($x=<STDIN>);
  if($x){
    push @configs,"# a directory into which a seperate html file for every different character of filenames is written";
    push @configs, join'',"html=".$x;
  }
  else{print "\n------> You have not specified any html file.\n";}
  
  &Line(); 
   
  # --m3u, specify m3u file
  print "\nYou can specify a m3u file to list your mp3 files in.";
  print "\nFor example:";
  print "\n     For Windows: c:\\windows\\Desktop\\myfiles.m3u";  
  print "\n     For Linux: /home/nikolei/myfiles.m3u";
  print "\nGive the name of the m3u file and it's absolute path:";
  chomp($x=<STDIN>);
  if($x){
    push @configs,"# the name of a playlistfile (e.g. for winamp)";
    push @configs, join'',"m3u=".$x;
  }
  else{print "\n------> You have not specified any m3u file.\n";}
  
  &Line(); 

  # --dbfile, specify a db file
  print "\nYou can specify a db file to list your file information in.";
  print "\nIn this file information is seperated by a special character.";
  print "\nFor example:";
  print "\n     For Windows: c:\\windows\\Desktop\\myfiles.db";  
  print "\n     For Linux: /home/nikolei/myfiles.db";
  print "\nNormally you would say no here.";
  print "\nGive the name of the db file and it's absolute path:";
  chomp($x=<STDIN>);
  if($x){
    push @configs,"# the name for a special db file";
    push @configs, join'',"dbfile=".$x;
  }
  else{print "\n------> You have not specified any db file.\n";}
  
  &Line(); 
  
  # --statfile, specify a html file for statistics
  print "\nYou can specify a html file for statistics about your files.";
  print "\nFor example:";
  print "\n     For Windows: c:\\windows\\Desktop\\statistics.html";  
  print "\n     For Linux: /home/nikolei/statistics.html";
  print "\nGive the name of the html file and it's absolute path:";
  chomp($x=<STDIN>);
  if($x){
    push @configs,"# the name for html file for summary statistics of the found files";
    push @configs, join'',"statfile=".$x;
  }
  else{print "\n------> You have not specified any stat file.\n";}
  
  &Line(); 
  
  # --hexfile, specify a hex file
  print "\nYou can specify a utf-8 file that will be used to replace"; 
  print "\nspecial characters.";
  print "\nFor example:";
  print "\n     For Windows: c:\\windows\\Desktop\\utf.dat";  
  print "\n     For Linux: /home/nikolei/utf.dat";
  print "\nGive the name of the hex file and it's absolute path:";
  chomp($x=<STDIN>);
  if($x){
    push @configs,"# the name of a file where utf code for replacements in urls is stored in";
    push @configs, join'',"utf8file=".$x;
  }
  else{print "\n------> You have not specified any hex file.\n";}
  
  &Line(); 

  # --replace, specify a replacement file
  print "\nYou can specify a replacement file for string replacements.";
  print "\nFor example:";
  print "\n     For Windows: c:\\windows\\Desktop\\replace.dat";
  print "\n     For Linux: /home/nikolei/replace.dat";
  print "\nGive the name of the replacement file and it's absolute path:";
  chomp($x=<STDIN>);
  if($x){
    push @configs,"# the name for a file where strings are stored in that should be replaced by other strings";
    push @configs, join'',"replace=".$x;
  }
  else{print "\n------> You have not specified any replacement file.\n";}
  
  &Line(); 
  
  # --sql, specify a sql file
  print "\nYou can specify a sql file to list your mp3 files in.";
  print "\nFor example:";
  print "\n     For Windows: c:\\windows\\Desktop\\myfiles.sql";  
  print "\n     For Linux: /home/nikolei/myfiles.sql";
  print "\nGive the name of the sql file and it's absolute path:";
  chomp($x=<STDIN>);
  if($x){
    push @configs,"# the name for a file to store data for a sql database in";
    push @configs, join'',"sql=".$x;
  }
  else{print "\n------> You have not specified any sql file.\n";}
  
  &Line(); 

  # --http
  print "\nYou can specify a url for your html files.";
  print "\nThe url will be added to the links for the files.";
  print "\nFor example:";
  print "\n     instead of <a href=\"/music/testfile.mp3\">";
  print "\n     you can define here http://mydomain.de and you will get";
  print "\n     <a href=\"http://mydomain.de/music/testfile.mp3\">";
  print "\nGive the name of the url:";
  chomp($x=<STDIN>);
  if($x){
    push @configs,"# predessor for the url in a html file, e.g. a domain name";
    push @configs, join'',"http=".$x;
  }
  else{print "\n------> You have not specified any url.\n";}
  
  &Line(); 
  
  # --remove
  print "\nYou can remove all id3 tags from your mp3/ogg files.";
  print "\nIf you want to do this, answer with yes, otherwise";
  print "\nwith no or just press RETURN. Normally you would answer";
  print "\nno here";
  print "\nTo avoid mistakes, if the option is set, it will be";
  print "\ncommented out in the config file, so you will have";
  print "\nedit it.";
  print "\nDo you want to remove all id3 tags? (yes,no):";
  chomp($x=<STDIN>);
  if($x=~/^(yes)|(y)$/){
    push @configs,"# remove all id3 tags";
    push @configs,"#remove";
  }
  else{print "\n-----> Id3 tags will not be removed.\n";}

  &Line();

  # --mp3info
  print "\nYou can use extendet information of  your";
  print "\nmp3 files for your html files.";
  print "\nThis information is about bitrate, freqency, length etc.";
  print "\nDo you want to use this information? (yes,no):";
  chomp($x=<STDIN>);
  if($x=~/^(yes)|(y)$/){
    push @configs,"# use mp3infos for html output";
    push @configs,"mp3info";
  }
  else{print "\n-----> Mp3 infos will not be used.\n";}

  &Line();

  # --dublicates
  print "\nYou can search for dublicates of file names.";
  print "\nDo you want to search for dublicates? (yes,no):";
  chomp($x=<STDIN>);
  if($x=~/^(yes)|(y)$/){
    push @configs,"# search for dublicates of file names";
    push @configs,"dublicates";
  }
  else{print "\n-----> Will not search for dublicates.\n";}
  
  &Line();

  # --fast
  print "\nYou can use a method for sorting faster.";
  print "\nThis is especially useful if you want to";
  print "\nindex a huge amount of files or mp3/ogg files";
  print "\nwith additional information. Under such conditions";
  print "\nthe method of fast sorting performs much better than";
  print "\nthe standard sorting. But it is experimental at the moment.";
  print "\nDo you want to use fast sorting? (yes,no):";
  chomp($x=<STDIN>);
  if($x=~/^(yes)|(y)$/){
    push @configs,"# use fast sorting (experimental)";
    push @configs,"fast";
  }
  else{print "\n-----> Will not use fast sorting.\n";}

  &Line();


  # --ext
  print "\nYou can remove the file extensions for displayed file names";
  print "\nin your html files.";
  print "\nDo you want to remove the file extension? (yes,no):";
  chomp($x=<STDIN>);
  if($x=~/^(yes)|(y)$/){
    push @configs,"# do not use file extensions for displayed filenames in html output";
    push @configs,"ext";
  }
  else{print "\n-----> File extensions will not be removed.\n";}

  &Line();

  # --filesize
  print "\nYou can use the file size of the files for your html files.";
  print "\nDo you want to use the filesize? (yes,no):";
  chomp($x=<STDIN>);
  if($x=~/^(yes)|(y)$/){
    push @configs,"# use the filesize for html output";
    push @configs,"filesize";
  }
  else{print "\n-----> File size will not be used.\n";}

  &Line();

  # --nocs
  print "\nYou can make the program to sort not case sensitive, so that";
  print "\nMYFILE and myfile are ranked equal within sorting.";
  print "\nDo you want to use not case sensitive sorting? (yes,no):";
  chomp($x=<STDIN>);
  if($x=~/^(yes)|(y)$/){
    push @configs,"# do filename sorting not case sensitive";
    push @configs,"nocs";
    print "\n------> The program will not sort case sensitive.\n";
  }
  else{print "\n-----> The program will sort case sensitive.\n";}

  &Line();

  # --rename
  print "\nYou can rename the files using their id3tag in a way that it becomes:";
  print "\nARTIST - ALBUM - TRACKNUMER - TITLE.mp3";
  print "\nDo you want to do that? (yes,no):";
  chomp($x=<STDIN>);
  if($x=~/^(yes)|(y)$/){
    push @configs,"# rename files using their id3tag";
    push @configs,"rename";
    print "\n------> The program will rename files using their id3tags.\n";
  }
  else{print "\n-----> The program will not rename files.\n";}

  &Line();

  # --renameback
  # --rename
  print "\nYou can rename the files back using the information in 'RENAME.bak'";
  print "\nso that they get their original file names back.";
  print "\nDo you want to do that? (yes,no):";
  chomp($x=<STDIN>);
  if($x=~/^(yes)|(y)$/){
    push @configs,"# rename files back using 'RENAME.bak'";
    push @configs,"renameback";
    print "\n------> The program will rename files back using their 'RENAME.bak'.\n";
  }
  else{print "\n-----> The program will not rename files back.\n";}

  &Line();

  # --skip
  print "\nYou skip one or more levels of the directory of a file";
  print "\nin the url in the html files. This option might usefull";
  print "\ntogether with the option http or if you define a special";
  print "\ndata directory for your files within the http server";
  print "\nFor example:";
  print "\n     skip=2 will make from c:\\music\\foo\\myfile.mp3";
  print "\n     <a href=\"/foo/myfile.mp3\">";
  print "\n     and will make from /home/nikolei/music/myfile.mp3";
  print "\n     <a href=\"/music/myfile.mp3\">";
  print "\nGive a value vor skip or just press RETURN:";
  chomp($x=<STDIN>);
  if($x){
    push @configs,"# skip the first n elements of the directory for urls in the html output";
    push @configs,"skip=".$x;
  }
  else{print "\n-----> The program will not use the skip option.\n";}

  &Line();

  # --check
  $x="yes";
  while($x=~ /^(y)|(yes)$/){
    print "\nYou can select file with specific file extensions.";
    print "\nFor example:";
    print "\n    mp3 and MP3 will match all files with these extensions";
    print "\n    like foo.mp3 and bar.MP3";
    print "\nGive a file extension:";
    chomp($x=<STDIN>);
    if($x){
      push @configs,"# only files with the following file extension are selected";
      push @configs, join '',"check=".$x;
    }
    print "\nDo you want to specify an other extension? (yes,no):";
    chomp($x=<STDIN>);
  }

  &Line();

  # --exec
  $x="yes";
  while($x=~ /^(y)|(yes)$/){
    print "\nYou can let the program execute other programs or scrips.";
    print "\nFor example you can copy files to an other locations";
    print "\nor whatever.";
    print "\n    For example: copy foo bar";
    print "\nGive the command to execute:";
    chomp($x=<STDIN>);
    if($x){
      push @configs,"# execute the following command";    
      push @configs, join '',"exec=".$x;
    }
    print "\nDo you want to specify an other command? (yes,no):";
    chomp($x=<STDIN>);
  }
  
  &Openfile($param,">",CONFFILE);
  foreach $configs(@configs){
    print {CONFFILE} "$configs\n";
  }
  &Closefile(CONFFILE);
    
  exit 1;
}



#********************************************************************************
# todo
# sql, own sort table, groupings
# option fuer direct write and buffered write!!!! sonst evtl speicherprobeme !!!!!!!!!!!!
# number of dirs
# set id3 tag with filename information
# beispiel fuer skip + http == samba
# SELECT name, directory FROM mp3table WHERE artist="A Guy Called Gerald" ORDER BY name LIMIT 10;
# SELECT CONCAT(directory,"\/",name)  FROM mp3table WHERE artist="A Guy Called Gerald" ORDER BY name LIMIT 10;
# SELECT CONCAT(directory,"\/",name)  FROM mp3table WHERE artist LIKE "%Called Gerald%" ORDER BY name LIMIT 10;
# SELECT directory, COUNT(*) AS count FROM mp3table GROUP BY directory ORDER BY count DESC;
# use REPLACEFLAG also for sql, db, m3u if availible
# use HEXTABLE also for sql should be optional (--> maybe not necessary, because php function)
# problems: a problem might occur in replacelink if there are spaces in strings!
# grouping: go through the hash and compare with group definitions, put all members of one group into
# a new hash, write them out, go to the next group.


